/*
    DataViewer visualization package
    Copyright (C) 1997 Randy Paffenroth and Thomas Stone

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
    MA 02111-1307, US`A


   Programmer: Ahmed M. Mentash & Dana Vrajitoru
   Program: This program is a Data viewer application that suppose to 
            animate a canon while firing a canon-ball into the air and
            then back to the ground.
*/

#include <iostream.h> 
#include <fstream.h>
#include <strstream.h>
#include <math.h>
#include "DV.h"
#include "DVrotation_interpolation_container.h"
#include "DVtranslation_interpolation_container.h"


// This animates the Earth on an ellipse of axes 6 and 9 around the Sun.
int Animate_canon_ball(DVcreate_image_arguments &dv);

int ament(DVcreate_image_arguments &dv)
{ 
  // Create 3 Containers to hold the canon parts.
  DVrotation_interpolation_container Canon, body, Wheels;
  
  // Create the canon's left Wheel
  DVlines Left_Wheel(2);
  Left_Wheel.set_line_status(DV_CYLINDER);
  Left_Wheel.width[0] = 2;
  Left_Wheel.set_translate(1.0,0.0,0.0);
  Left_Wheel.color[0]=DVcolor(1, 0.5, 0);
  Left_Wheel.set_rotate(0,0,0,90);
 
  // Create the canon's right Wheel
  DVlines Right_Wheel(2);
  Right_Wheel.set_line_status(DV_CYLINDER);
  Right_Wheel.width[0] = 2;
  Right_Wheel.set_translate(1.0,-2.0,0.0);
  Right_Wheel.color[0]=DVcolor(1, 0.5, 0);
  Right_Wheel.set_rotate(0,0,0,90);
  
  // Copy the wheels of the canon into the Wheels container
  Wheels.copy_child(Right_Wheel);
  Wheels.copy_child(Left_Wheel);

  // Create the canon's Pipe
  DVlines Pipe(4);
  Pipe.set_line_status(DV_CYLINDER);
  Pipe.width[0] = 0.5;
  Pipe.set_translate(1.0,-1.0,0.0);
  Pipe.set_rotate(0,0,0,180);
  Pipe.set_translate(5.0,0.5,0.0);
  Pipe.color[0]=DVcolor(0.0, 0.5, 0.0);
  
  // Create a sphere that lies at the end of the canon pipe 
  DVspheres End_Pipe(0.0, 0.0, 0.0,1.0);
  End_Pipe.set_translate(0.5,0.5,0.0);
  End_Pipe.color[0]=DVcolor(0.0, 0.5, 0.0);

  // Copy the Pipe and the sphere at the end of it into the body container
  body.copy_child(Pipe);
  body.copy_child(End_Pipe);
 
  // Copy the Wheels and the body containers into the Canon Container
  Canon.copy_child(Wheels);
  Canon.copy_child(body);

  Canon.set_translate(-2.7,-0.5,7.0);
  Canon.set_rotate(0,1,0,15);

  // Now we copy the Whole canon into the "dv.top" node
  dv.top.copy_child(Canon);
  

  // Create a reference plane
  DVboxes plane(-10, 0, -10, 10, 0, 10);
  plane.set_translate(-0.2,0.0,-5.0);
  plane.set_rotate(0,0,0,90);
  dv.top.copy_child(plane);

  // Calling the function that will animate the canon action
  Animate_canon_ball(dv);

  dv.top.geometry_complexity = 9;

  // Set the frames to be drawn.
  dv.dvhints.set_animation_first_frame_number(0); 
  dv.dvhints.set_animation_last_frame_number(40); 

  return 1;
}



// This function creates an animation based on a sphere with fix position
// using a rotation and a translation interpolation containers to create an elliptical trajectory.
int Animate_canon_ball(DVcreate_image_arguments &dv)
{
  int i, nr_key_frames = 12;
  DVspheres canon_ball(0.0, 0.0, 0.0, 0.25);
  canon_ball.color[0]=DVcolor(0.4, 0.4, 1);

  DVtranslation_interpolation_container trans_anim;

  // Set the frames
  LCVMarray<int> the_frames(nr_key_frames);
  for (i=0; i<nr_key_frames; i++)
    the_frames[i] = i * 20;

 // Set the translation vectors
  LCVMarray_2D<float> the_vectors(nr_key_frames, 3);
  for (i=0; i<nr_key_frames; i++) {
    the_vectors[i][0] = 0.0;
    the_vectors[i][1] = 0.0;
    if (i%2)
      the_vectors[i][2] = 9.0;
    else
      the_vectors[i][2] = 6.0;
  }

  // Apply this data to trans_anim
  trans_anim.set_data(the_frames, the_vectors);

  // Attach the canon_ball to it
  trans_anim.copy_child(canon_ball);

  DVrotation_interpolation_container rotate_anim;

  // We'll use the same key frames as for the previous container

  // Set the rotation axes : constant for all the frames
  LCVMarray_2D<float> the_axes(nr_key_frames, 3);
  for (i=0; i<nr_key_frames; i++) {
    the_axes[i][0] = 0.0;
    the_axes[i][1] = 50.0;
    the_axes[i][2] = 0.0;
  }

  // Set the rotation angle so that it complete one circle in 5 key-frames
  LCVMarray<float> the_angles(nr_key_frames);
  for (i=0; i<nr_key_frames; i++)
    the_angles[i] = i * 90.0;

  // Send all this data to the container
  rotate_anim.set_data(the_frames, the_axes, the_angles);

  // Attach the previous container to it.
  rotate_anim.copy_child(trans_anim);

  // Attach the whole thing to dv.top
  dv.top.copy_child(rotate_anim);
}












