/*
    DataViewer visualization package
    Copyright (C) 1997 Randy Paffenroth and Thomas Stone and Dana Vrajitoru

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
    MA 02111-1307, USA
*/

/* 
  Application written by D. Vrajitoru simulating a canon, for the
  Image Synthesis class, September 2003.  
*/

#include <iostream.h> 
#include <fstream.h>
#include <strstream.h>
#include <math.h>
#include "DV.h"
#include "DVtranslation_interpolation_container.h"
#include "canon.h"

// Function drawing a canon and throwing a ball from it.
int canon(DVcreate_image_arguments &dv)
{
  istrstream top_fin((char const*)dv.input_string,
		     dv.input_string.getLength());
  float angle, speed;
  int frames;
  
  // Retrieve the parameters from the input string
  top_fin >> angle >> speed;

  angle = M_PI * angle / 180.0;

  make_canon(dv.top, angle);
  dv.top.geometry_complexity = 9;

  frames = animate_ball(dv.top, angle, speed);

  // Set the frames to be drawn.
  dv.dvhints.set_animation_first_frame_number(0); 
  dv.dvhints.set_animation_last_frame_number(frames-1);
 
  return 1;
}

// Makes a ball and projects it.
int animate_ball(DVcontainerbase &top, float angle, float speed)
{
  const float s_scale = 0.5, dt=0.1, g = 9.8;
  float v0x, v0y, ttime, x, y, t, vy, deltat;
  int nr_frames;

  DVspheres ball(0.95);
  ball.color[0]=DVcolor(0.7, 0.7, 0.7);

  
  // An animated container that will contain the Earth at different positions
  DVselection_container ball_move;
  //DVcontainerbase ball_move;

  v0x = speed * cos(angle);
  v0y = speed * sin(angle);
  ttime = 2*v0y / g;
  nr_frames = int(ttime / dt) + 2;
  //cout << ttime << ' ' << nr_frames << endl;
 
  for (int i=0; i<nr_frames; i++) {
    t = dt * i;
    x = v0x * t;
    vy = v0y - g*t;
    y = v0y * t - 0.5 * g * t*t;
    //cout << t << ' ' << vy << ' ' << y << endl;
    ball.set_translate(x, y, 0);
    ball_move.copy_child(ball);
  }
  top.copy_child(ball_move);
  return nr_frames;
}

// Draws the canon itself.
void make_canon(DVcontainerbase &top, float angle)
{
  DVcontainerbase wheel_ct;
  wheel(wheel_ct, 2.0);
  wheel_ct.set_translate(0, 0, -1);
  top.copy_child(wheel_ct);
  wheel_ct.set_translate(0, 0, 1);
  top.copy_child(wheel_ct);
  DVspheres center;
  top.copy_child(center);
  DVlines barrel(0, 0, 0, 3.0*cos(angle), 3.0*sin(angle), 0);
  barrel.width[0]=1;
  barrel.set_line_status(DV_CYLINDER);
  top.copy_child(barrel);
  top.color[0] = DVcolor(0.6, 0.6, 0.6);

  DVboxes floor(-3, -2, -3, 50, -2, 3);
  floor.color[0] = DVcolor(0.6, 0.8, 0.6);
  top.copy_child(floor);
}

// Draws a wheel with a given radius. Centered in the origin, vertical
// axis: y, horizontal axis: x. The wheel rotates around z.
void wheel(DVcontainerbase &parent, float radius)
{
  int precision = 30, speed = 2, i;
  LCVMarray_2D<float> wheel_coord(precision+1, 3);
  float alpha = 2*M_PI/precision, tire_width = 0.18, spike_width = 0.07, 
    ax_width = 0.12, ax_len = 0.4;
  for (i=0; i<=precision; i++) {
    wheel_coord[i][0]=radius*cos(i*alpha);
    wheel_coord[i][1]=radius*sin(i*alpha);
    wheel_coord[i][2]=0;
  }
  DVlines tire(wheel_coord);
  tire.set_line_status(DV_CYLINDER);
  tire.width[0]=tire_width;
  tire.color[0]=DVcolor(0.5, 0.5, 0.6);
  alpha *= 3;
  precision = precision / 3;
  LCVMarray_3D<float> spike_coord(precision, 2, 3);
  for (i=0; i<precision; i++) {
    spike_coord[i][0][0] = 0;
    spike_coord[i][0][1] = 0;
    spike_coord[i][0][2] = 0;
    spike_coord[i][1][0] = radius*cos(i*alpha);
    spike_coord[i][1][1] = radius*sin(i*alpha);
    spike_coord[i][1][2] = 0;
  }
  DVlines spike(spike_coord);
  spike.set_line_status(DV_CYLINDER);
  spike.width[0]=spike_width;
  parent.copy_child(tire);
  parent.copy_child(spike);

  DVlines ax(0, 0, -ax_len, 0, 0, ax_len);
  ax.set_line_status(DV_CYLINDER);
  ax.width[0]=ax_width;
  parent.copy_child(ax);

  DVspheres axs1(0, 0, -ax_len, ax_width), axs2(0, 0, ax_len, ax_width);
  parent.copy_child(axs1);
  parent.copy_child(axs2);
  parent.color[0]=DVcolor(0.5, 0.5, 0.6);
}
