/*
    DataViewer visualization package
    Copyright (C) 1997 Randy Paffenroth, Thomas Stone, and Dana Vrajitoru

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
    MA 02111-1307, USA
*/

#include <iostream.h> 
#include <fstream.h>
#include <strstream.h>
#include <math.h>
#include "DV.h"
#include "DVrotation_interpolation_container.h"
#include "DVtranslation_interpolation_container.h"

// This animates the Earth on a circle of radius 6 around the Sun.
int Animate_earth_circle(DVcreate_image_arguments &dv);

// This animates the Earth on an ellipse of axes 6 and 9 around the Sun.
int Animate_earth_ellipse(DVcreate_image_arguments &dv);

int Animate_moon_circle(DVcontainerbase &parent);
int Animate_moon_ellipse(DVcontainerbase &parent);

int earth_moon(DVcreate_image_arguments &dv)
{ 
  int method=0;
  istrstream top_fin((char const*)dv.input_string,dv.input_string.getLength());
  top_fin >> method;

  // Create the sun as an orange sphere of radius 1, centered in the origin.
  DVspheres sun(0, 0, 0, 0.8);
  sun.color[0]=DVcolor(1, 0.5, 0);
  dv.top.copy_child(sun);

  // Create a reference plane
  DVboxes plane(-10, 0, -10, 10, 0, 10);
  dv.top.copy_child(plane);
  
  if (method == 0)
    Animate_earth_circle(dv);
  else
    Animate_earth_ellipse(dv);

  dv.top.geometry_complexity = 9;

  // Set the frames to be drawn.
  dv.dvhints.set_animation_first_frame_number(0); 
  dv.dvhints.set_animation_last_frame_number(160); 

  return 1;
}

// This function creates an animation based on a sphere with fix position
// using a rotation interpolation container.
int Animate_earth_circle(DVcreate_image_arguments &dv)
{
  int i, nr_key_frames = 9;

  DVspheres earth(0, 0, 0, 0.45);
  earth.color[0]=DVcolor(0.4, 0.4, 1);

  DVcontainer trans_node;
  trans_node.copy_child(earth);
  trans_node.set_translate(6.0, 0.0, 0.0);

  DVrotation_interpolation_container rotate_anim;

  // Set the frames
  LCVMarray<int> the_frames(nr_key_frames);
  for (i=0; i<nr_key_frames; i++)
    the_frames[i] = i * 20;

  // Set the rotation axes : constant for all the frames
  LCVMarray_2D<float> the_axes(nr_key_frames, 3);
  for (i=0; i<nr_key_frames; i++) {
    the_axes[i][0] = 0.0;
    the_axes[i][1] = 50.0;
    the_axes[i][2] = 0.0;
  }

  // Set the rotation angle so that it complete one circle in 5 key-frames
  LCVMarray<float> the_angles(nr_key_frames);
  for (i=0; i<nr_key_frames; i++)
    the_angles[i] = i * 90.0;
  
  // Send all this data to the container
  rotate_anim.set_data(the_frames, the_axes, the_angles);

  Animate_moon_circle(trans_node);

  // Attach the previous container to it.
  rotate_anim.copy_child(trans_node);

  // Attach the whole thing to dv.top
  dv.top.copy_child(rotate_anim);
}

// This function creates an animation based on a sphere with fix position
// using a rotation interpolation container.
int Animate_moon_circle(DVcontainerbase &parent)
{
  int i, nr_key_frames = 33;

  DVspheres moon(0.0, 0.0, 0.0, 0.2);
  moon.color[0]=DVcolor(0.7, 0.7, 0.8);

  DVcontainer trans_node;
  trans_node.copy_child(moon);
  trans_node.set_translate(2.0, 0.0, 0.0);

  DVrotation_interpolation_container rotate_anim;

  // Set the frames
  LCVMarray<int> the_frames(nr_key_frames);
  for (i=0; i<nr_key_frames; i++)
    the_frames[i] = i * 5;

  // Set the rotation axes : constant for all the frames
  LCVMarray_2D<float> the_axes(nr_key_frames, 3);
  for (i=0; i<nr_key_frames; i++) {
    the_axes[i][0] = 0.0;
    the_axes[i][1] = 50.0;
    the_axes[i][2] = 0.0;
  }

  // Set the rotation angle so that it complete one circle in 5 key-frames
  LCVMarray<float> the_angles(nr_key_frames);
  for (i=0; i<nr_key_frames; i++)
    the_angles[i] = i * 90.0;
  
  // Send all this data to the container
  rotate_anim.set_data(the_frames, the_axes, the_angles);

  // Attach the previous container to it.
  rotate_anim.copy_child(trans_node);

  // Attach the whole thing to dv.top
  parent.copy_child(rotate_anim);
}

// This function creates an animation based on a sphere with fix position
// using a rotation and a translation interpolation containers to create an elliptical trajectory.
int Animate_earth_ellipse(DVcreate_image_arguments &dv)
{
  int i, nr_key_frames = 9;
  DVspheres earth(0.0, 0.0, 0.0, 0.45);
  earth.color[0]=DVcolor(0.4, 0.4, 1);

  DVtranslation_interpolation_container trans_anim;

  // Set the frames
  LCVMarray<int> the_frames(nr_key_frames);
  for (i=0; i<nr_key_frames; i++)
    the_frames[i] = i * 20;

 // Set the translation vectors
  LCVMarray_2D<float> the_vectors(nr_key_frames, 3);
  for (i=0; i<nr_key_frames; i++) {
    the_vectors[i][0] = 0.0;
    the_vectors[i][1] = 0.0;
    if (i%2)
      the_vectors[i][2] = 6.0;
    else
      the_vectors[i][2] = 9.0;
  }

  // Apply this data to trans_anim
  trans_anim.set_data(the_frames, the_vectors);

  // Attach the Earth to it
  trans_anim.copy_child(earth);

  DVrotation_interpolation_container rotate_anim;

  // We'll use the same key frames as for the previous container

  // Set the rotation axes : constant for all the frames
  LCVMarray_2D<float> the_axes(nr_key_frames, 3);
  for (i=0; i<nr_key_frames; i++) {
    the_axes[i][0] = 0.0;
    the_axes[i][1] = 50.0;
    the_axes[i][2] = 0.0;
  }

  // Set the rotation angle so that it complete one circle in 5 key-frames
  LCVMarray<float> the_angles(nr_key_frames);
  for (i=0; i<nr_key_frames; i++)
    the_angles[i] = i * 90.0;

  // Send all this data to the container
  rotate_anim.set_data(the_frames, the_axes, the_angles);

  Animate_moon_ellipse(trans_anim);

  // Attach the previous container to it.
  rotate_anim.copy_child(trans_anim);

  // Attach the whole thing to dv.top
  dv.top.copy_child(rotate_anim);
}

// This function creates an animation based on a sphere with fix position
// using a rotation and a translation interpolation containers to create an elliptical trajectory.
int Animate_moon_ellipse(DVcontainerbase &parent)
{
  int i, nr_key_frames = 33;
  DVspheres moon(0.0, 0.0, 0.0, 0.2);
  moon.color[0]=DVcolor(0.7, 0.7, 0.8);

  DVtranslation_interpolation_container trans_anim;

  // Set the frames
  LCVMarray<int> the_frames(nr_key_frames);
  for (i=0; i<nr_key_frames; i++)
    the_frames[i] = i * 5;

 // Set the translation vectors
  LCVMarray_2D<float> the_vectors(nr_key_frames, 3);
  for (i=0; i<nr_key_frames; i++) {
    the_vectors[i][0] = 0.0;
    the_vectors[i][1] = 0.0;
    if (i%2)
      the_vectors[i][2] = 2.0;
    else
      the_vectors[i][2] = 1.5;
  }

  // Apply this data to trans_anim
  trans_anim.set_data(the_frames, the_vectors);

  // Attach the Moon to it
  trans_anim.copy_child(moon);

  DVrotation_interpolation_container rotate_anim;

  // We'll use the same key frames as for the previous container

  // Set the rotation axes : constant for all the frames
  LCVMarray_2D<float> the_axes(nr_key_frames, 3);
  for (i=0; i<nr_key_frames; i++) {
    the_axes[i][0] = 0.0;
    the_axes[i][1] = 50.0;
    the_axes[i][2] = 0.0;
  }

  // Set the rotation angle so that it complete one circle in 5 key-frames
  LCVMarray<float> the_angles(nr_key_frames);
  for (i=0; i<nr_key_frames; i++)
    the_angles[i] = i * 90.0;

  // Send all this data to the container
  rotate_anim.set_data(the_frames, the_axes, the_angles);

  // Attach the previous container to it.
  rotate_anim.copy_child(trans_anim);

  // Attach the whole thing to dv.top
  parent.copy_child(rotate_anim);
}
