/*
    DataViewer visualization package
    Copyright (C) 1997 Randy Paffenroth and Thomas Stone

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
    MA 02111-1307, USA
*/

#include <iostream.h> 
#include <fstream.h>
#include <strstream.h>
#include <math.h>
#include "parameters.h"
#include "DV.h"

const float a=1.0, c=2.0;
int parameters(DVcreate_image_arguments &dv)
{ 
  istrstream top_fin((char const*)dv.input_string,
		     dv.input_string.getLength());
  int spoints, tpoints;
  
  // Retrieve the parameters from the input string
  top_fin >> spoints >> tpoints;

  Discretize_sphere(dv, spoints, tpoints, 0);

  dv.top.geometry_complexity = 4;
  dv.top.light_complexity = (int) 9;
  return 1;
}

// This function discretizes a 3D surface and draws it as a set of triangles
// if to_strip is 0, and to triangle strips if the to_strips is 1
int Discretize_sphere(DVcreate_image_arguments &dv, 
                     int spoints, int tpoints, int to_strips)
{
  const float a=0.0, b=2*M_PI, c=0, d=2*M_PI;
  float s1, s2, t1, t2;
  float sstep = (b - a) / spoints; 
  float tstep = (d - c) / tpoints; 

  // To create a surface by triangle strips, we use the class DVtriangle_grid
  // that takes a list of points organized in a grid and creates a list of
  // triangle strips from it. See the description of the class for more details.
  if (to_strips) {
      LCVMarray_3D<float> points(spoints+1, tpoints+1, 3);

    for (int i=0; i<=spoints; i++)
      for (int j=0; j<=tpoints; j++) {
        s1 = a + i * sstep;
        t1 = c + j * tstep;

        Copy_point(points[i][j], point(s1, t1));
      }

    // Create a sphere from the array of points
    DVtriangle_grid sphere(points);

    // Uncomment this to color the sphere with a color map according to the x
    // coordinate.
    //sphere.set_color_coordinate(0);

    // Set its color to yellow
    sphere.color[0] = DVcolor(1.0, 1.0, 0.0);

    // Attach it to the root of the tree.
    dv.top.copy_child(sphere);
  }
  else {
    LCVMarray_3D<float> points(2*spoints*tpoints, 3, 3);

    for (int i=0; i<spoints; i++) 
      for (int j=0; j<tpoints; j++) { 
        s1 = a + i * sstep; 
        s2 = a + (i+1) * sstep; 
        t1 = c + j * tstep; 
        t2 = c + (j+1) * tstep; 

        Triangle(points[2*(i*tpoints+j)], 
                 point(s1, t1), point(s1, t2), point(s2, t1)); 
        Triangle(points[2*(i*tpoints+j)+1],
                 point(s2, t1), point(s1, t2), point(s2, t2)); 
      }

    // Create a sphere from the array of points
    DVtriangle_strips sphere(points);

    // Set its color to yellow
    sphere.color[0] = DVcolor(1.0, 1.0, 0.0);

    // Uncomment this to color the sphere with a color map according to the x
    // coordinate.
    //sphere.set_color_coordinate(0);

    // Attach it to the root of the tree.
    dv.top.copy_child(sphere);
  }
}

// This function creates a triangle from 3 points
void Triangle(LCVMarray_2D<float> &where, float *p1, float *p2, float *p3)
{
  Copy_point(where[0], p1);
  Copy_point(where[1], p2);
  Copy_point(where[2], p3); 
}

// Copy a normal array to an LCVMarray
void Copy_point(LCVMarray<float> &lcvm_point, float *normal_point)
{
  for (int i=0; i<3; i++)
    lcvm_point[i] = normal_point[i];
}

// Compute the point on the surface corresponding to s and t
float *point(float s, float t)
{
  float *pt = new float[3];
  pt[0] = X(s, t);
  pt[1] = Y(s, t);
  pt[2] = Z(s, t);
  return pt;
}

// The X component of the point on the surface corresponding to s and t
float X(float s, float t)
{
  //return cos(s)*cos(t);
  return (c + a*cos(t)) * cos(s);
}

// The Y component of the point on the surface corresponding to s and t
float Y(float s, float t)
{
  //return sin(s)*cos(t);
  return a * sin(t);
  //return (c + a*cos(t)) * sin(s);
}

// The Z component of the point on the surface corresponding to s and t
float Z(float s, float t)
{
  //return sin(t);
  //return a * sin(t);
  return a*sin(s);
}
