/*
    DataViewer visualization package
    Copyright (C) 1997 Randy Paffenroth and Thomas Stone and Dana Vrajitoru

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
    MA 02111-1307, USA
*/

/* 
  Application written by D. Vrajitoru simulating a gravitational
  pendulum, for the Image Synthesis class, September 2003.  
*/

#include <iostream.h> 
#include <fstream.h>
#include <strstream.h>
#include <math.h>
#include "DV.h"
#include "DVtranslation_interpolation_container.h"
#include "DVrotation_interpolation_container.h"
#include "pendulum.h"

// Some dimensions to be used by these functions.
const float bar_length = 3.0, section_radius = 0.2, ball_radius = 0.5;
const float frame_height = 5.0, frame_width = 2.5, frame_length = 3;
const float dt=0.1;

// Simulates and animates a gravitational pendulum.
int pendulum(DVcreate_image_arguments &dv)
{  
  float angle;
  int frames;
  
  istrstream top_fin((char const*)dv.input_string,
		     dv.input_string.getLength());
  // Retrieve the angle from the input string
  top_fin >> angle;
  //angle_r = M_PI * angle_d / 180.0;
  
  //DVcontainerbase pendulum_ct;
  //make_pendulum(pendulum_ct);
  //pendulum_ct.set_rotate(0, 0, 1, angle);
  //dv.top.copy_child(pendulum_ct);

  make_frame(dv.top);

  frames = animate_pendulum(dv.top, angle);

  // Set the frames to be drawn.
  dv.dvhints.set_animation_first_frame_number(0); 
  dv.dvhints.set_animation_last_frame_number(2*frames-1);

  dv.top.geometry_complexity = 9;
  return 1;
}

// Animate the pendulum with a constant speed of 20 degrees by second
int animate_pendulum(DVcontainerbase &top, float start_angle)
{
  DVrotation_interpolation_container swing;

  // Attach the pendulum to this container.
  make_pendulum(swing);

  int i;
  float speed = 20; // degrees by second

  // number of frames to be drawn. We have a key frame for every
  // frame, so there's no interpolation.
  int nr_key_frames = 4 * int( start_angle / (dt * speed));
  LCVMarray<int> the_frames(nr_key_frames);
  for (i=0; i<nr_key_frames; i++)
    the_frames[i] = i;
  
  // Set the rotation axis : constant for all the frames
  LCVMarray_2D<float> the_axes(nr_key_frames, 3);
  for (i=0; i<nr_key_frames; i++) {
    the_axes[i][0] = 0.0;
    the_axes[i][1] = 0.0;
    the_axes[i][2] = 1.0;
  }

  // Set the rotation angle to be the speed times the time for each
  // frame. The time is dt times the frame number.
  LCVMarray<float> the_angles(nr_key_frames);
  for (i=0; i<nr_key_frames; i++)
    if (i < nr_key_frames/2)
      the_angles[i] = start_angle - speed * i * dt;
    else
      the_angles[i] = start_angle - speed * (nr_key_frames - i) * dt;

  // Set the container's data with the nr of frames, rotation axis and
  // angles and make it loop over this animation.
  swing.set_data(the_frames, the_axes, the_angles);
  swing.set_animation_status(DV_LOOP);
  
  // Add the animation container to the cop container.
  top.copy_child(swing);

  // Return the number of frames such that the parent function knows
  // how many frames must be drawn on the whole.
  return nr_key_frames;
}

// Generates the pendulum itself which is a sphere on top of a
// cylinder.
void make_pendulum(DVcontainerbase &top)
{
  // Make a vertical cylinder
  DVlines bar(0, 0, 0, 0, -bar_length, 0);
  bar.width[0]=section_radius;
  bar.set_line_status(DV_CYLINDER);
  top.copy_child(bar);

  // Top the cylinder with a small sphere
  DVspheres bar_top(1.2*section_radius);
  top.copy_child(bar_top);

  // A sphere of radius ball_radius, centered in the origin.
  DVspheres ball(ball_radius);
  // Move it at the end of the cylinder.
  ball.set_translate(0, -bar_length, 0);
  top.copy_child(ball);
}

// Makes a frame around the pendulum.
void make_frame(DVcontainerbase &top)
{
  // Coordinates of the frame
  float frame_coords[5][3] = {
    {0, 0, -frame_width}, {0, 0, frame_width},
    {0, -frame_height, frame_width}, {0, -frame_height, -frame_width}, 
    {0, 0, -frame_width}};
  LCVMarray_2D<float> frame_lcvm_coords(5, 3);
  for (int i=0; i<5; i++)
    for (int j=0; j<3; j++)
      frame_lcvm_coords[i][j] = frame_coords[i][j];
  // Make a bunch of cylinders with these coordinates.
  DVlines frame(frame_lcvm_coords);
  frame.width[0]=section_radius;
  frame.set_line_status(DV_CYLINDER);
  top.copy_child(frame);

  // Close the upper parts with spheres of the same radius.
  DVspheres corner(section_radius);
  corner.set_translate(frame_coords[0][0], 
		       frame_coords[0][1], 
		       frame_coords[0][2]);
  top.copy_child(corner);
  // Translate the same object to the other side and attach it to the
  // top container again.
  corner.set_translate(frame_coords[1][0], 
		       frame_coords[1][1], 
		       frame_coords[1][2]);
  top.copy_child(corner);

  // Make a box for the frame to sit on. The height of the box is the
  // same as the diameter of the frame cylinders.
  DVboxes ground(-frame_length-section_radius, -frame_height, -frame_width,
		 frame_length, -frame_height-2*section_radius, 
		 frame_width+section_radius);
  top.copy_child(ground);
}
